@@ -43,9 +43,9 @@ HTMLParser::HTMLParser(const char * wordchars)
 	init(wordchars);
 }
 
-HTMLParser::HTMLParser(struct unicode_info2 * uconv, unsigned short * wordchars, int len)
+HTMLParser::HTMLParser(unsigned short * wordchars, int len)
 {
-	init(uconv, wordchars, len);
+	init(wordchars, len);
 }
 
 HTMLParser::~HTMLParser() 
