@@ -55,9 +55,9 @@ TextParser::TextParser(const char * wordchars)
 	init(wordchars);
 }
 
-TextParser::TextParser(struct unicode_info2 * uconv, unsigned short * wordchars, int len)
+TextParser::TextParser(unsigned short * wordchars, int len)
 {
-	init(uconv, wordchars, len);
+	init(wordchars, len);
 }
 
 TextParser::~TextParser() 
@@ -72,7 +72,7 @@ int TextParser::is_wordchar(char * w)
                 unsigned short idx;
 		u8_u16(&wc, 1, w);
                 idx = (wc.h << 8) + wc.l;
-                return (utfconv[idx].cletter || (wordchars_utf16 && flag_bsearch(wordchars_utf16, *((unsigned short *) &wc), wclen)));
+                return (unicodeisalpha(idx) || (wordchars_utf16 && flag_bsearch(wordchars_utf16, *((unsigned short *) &wc), wclen)));
         } else {
 		return wordcharacters[(*w + 256) % 256];
 	}
@@ -99,7 +99,6 @@ void TextParser::init(const char * wordchars)
 	token = 0;
 	state = 0;
         utf8 = 0;
-        utfconv = NULL;
 	unsigned int j;
 	for (j = 0; j < 256; j++)
 		wordcharacters[j] = 0;
@@ -109,7 +108,7 @@ void TextParser::init(const char * wordchars)
 	}
 }
 
-void TextParser::init(struct unicode_info2 * uconv, unsigned short * wc, int len)
+void TextParser::init(unsigned short * wc, int len)
 {
 	for (int i = 0; i < MAXPREVLINE; i++) {
 		line[i][0] = '\0';
@@ -119,7 +118,6 @@ void TextParser::init(struct unicode_info2 * uconv, unsigned short * wc, int len
 	token = 0;
 	state = 0;
 	utf8 = 1;
-        utfconv = uconv;
         wordchars_utf16 = wc;
         wclen = len;
 }
