@@ -11,10 +11,14 @@
 #include "config.h"
 #include "hunspell.hxx"
 
+#ifndef HUNSPELL_EXTRA
+#define suggest_auto suggest
+#endif
+
 #define HUNSPELL_VERSION VERSION
 #define INPUTLEN 50
 
-#define HUNSPELL_PIPE_HEADING "@(#) International Ispell Version 3.2.06 (but really Hunspell HUNSPELL_VERSION)"
+#define HUNSPELL_PIPE_HEADING "@(#) International Ispell Version 3.2.06 (but really Hunspell "VERSION")"
 #define HUNSPELL_HEADING "Hunspell "
 
 //for debugging only
@@ -126,12 +130,13 @@ int unicode = 0;
 
 TextParser * newParser(char * wordchars, int format, char * extension, Hunspell * pMS) {
     TextParser * p = NULL;
+    int utf8 = (strcmp(pMS->get_dic_encoding(), "UTF-8") == 0);
 
-    if (pMS->get_utf_conv()) {
+    if (utf8) {
         switch (format) {	
-        case FMT_LATEX: p = new LaTeXParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len); break;
-        case FMT_HTML: p = new HTMLParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len); break;
-        case FMT_MAN: p = new ManParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len); break;
+        case FMT_LATEX: p = new LaTeXParser(wordchars_utf16, wordchars_utf16_len); break;
+        case FMT_HTML: p = new HTMLParser(wordchars_utf16, wordchars_utf16_len); break;
+        case FMT_MAN: p = new ManParser(wordchars_utf16, wordchars_utf16_len); break;
         case FMT_FIRST: p = new FirstParser(wordchars);
         }
     } else {
@@ -147,28 +152,28 @@ TextParser * newParser(char * wordchars, int format, char * extension, Hunspell
 	if ((strcmp(extension, "html") == 0) ||
     	    (strcmp(extension, "htm") == 0) ||
 	    (strcmp(extension, "xml") == 0)) {
-                if (pMS->get_utf_conv()) {
-                    p = new HTMLParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len);
+                if (utf8) {
+                    p = new HTMLParser(wordchars_utf16, wordchars_utf16_len);
                 } else {
 		    p = new HTMLParser(wordchars);
                 }
 	} else if (((extension[0] > '0') && (extension[0] <= '9'))) {
-                if (pMS->get_utf_conv()) {
-	            p = new ManParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len);
+                if (utf8) {
+	            p = new ManParser(wordchars_utf16, wordchars_utf16_len);
                 } else {
 	            p = new ManParser(wordchars);
                 }
 	} else if ((strcmp(extension, "tex") == 0)) {
-                if (pMS->get_utf_conv()) {
-                    p = new LaTeXParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len);
+                if (utf8) {
+                    p = new LaTeXParser(wordchars_utf16, wordchars_utf16_len);
                 } else {
 	            p = new LaTeXParser(wordchars);
                 }
 	}
     }
     if (!p) {
-        if (pMS->get_utf_conv()) {
-            p = new TextParser(pMS->get_utf_conv(), wordchars_utf16, wordchars_utf16_len);
+        if (utf8) {
+            p = new TextParser(wordchars_utf16, wordchars_utf16_len);
         } else {
             p = new TextParser(wordchars);    
         }
@@ -204,7 +209,7 @@ int putdic(char * word, Hunspell * pMS)
 	if (*(w+1) == '/') {
 	    ret = pMS->put_word_pattern(word, w + 2); // word//pattern
 	} else {
-	    ret = pMS->put_word_suffix(word, w + 1); // word/flags
+	    ret = pMS->put_word_pattern(word, w + 1); // word/pattern
 	}
 	*w = c;
 	return ret;
@@ -256,7 +261,6 @@ int save_privdic(char * filename, char * filename2, wordlist * w)
 	free(r);
     }
     fclose(dic);
-    return 1;
 }
 
 char * basename(char * s, char c) {
@@ -285,6 +289,7 @@ char * token;
 int pos;
 int bad;
 int lineno = 0;
+int terse_mode = 0;
 
 TextParser * parser = newParser(wordchars, format, NULL, pMS);
 
@@ -313,7 +318,8 @@ nextline: while(fgets(buf, MAXLNLEN, fileid)) {
     if (filter_mode == PIPE) {
     pos = -1;
     switch (buf[0]) {
-    case '!': { break; }
+    case '%': { terse_mode = 0; break; }
+    case '!': { terse_mode = 1; break; }
     case '+': {
 		delete parser;
 		parser = new LaTeXParser(wordchars);
@@ -405,7 +411,7 @@ if (pos >= 0) {
 		case AUTO2: {
                 FILE * f = (filter_mode == AUTO) ? stderr : stdout;
 		if (! pMS->spell(token)) {
-			char ** wlst;
+			char ** wlst = NULL;
 			bad = 1;
 			int ns = pMS->suggest_auto(&wlst, token);
 			if (ns > 0) {
@@ -429,20 +435,21 @@ if (pos >= 0) {
 
 		case PIPE:
 		case NORMAL: {
-		
-		if (pMS->spell(token)) {
+		int info;
+                char * root = NULL;
+		if (pMS->spell(token, &info, &root)) {
 			if (filter_mode == PIPE) {
-				fprintf(stdout,"*\n");
-			} else if (pMS->get_prevcompound()) {
+				if (!terse_mode) fprintf(stdout,"*\n");
+			} else if (info & SPELL_COMPOUND) {
 				fprintf(stdout,"-\n");
-			} else if (pMS->get_prevroot()) {
-		    		fprintf(stdout,"+ %s\n", pMS->get_prevroot());
+			} else if (root) {
+		    		fprintf(stdout,"+ %s\n", root);
 			} else {
 				fprintf(stdout,"*\n");
 			}			
 			fflush(stdout);
 		} else {
-			char ** wlst;
+			char ** wlst = NULL;
 			int ns = pMS->suggest(&wlst, token);
 			if (ns == 0) {
 		    		fprintf(stdout,"# %s %d", token, parser->get_tokenpos() + pos);
@@ -575,9 +582,12 @@ void dialogscreen(TextParser * parser, char * token,
 		free(line);
 	}
 
+	char * line = (char *) calloc(1, x + 1);
+	strncpy(line, lines[0] + x * rowindex, parser->get_tokenpos() % x);
+	mvprintw(MAXPREVLINE + 1 - beginrow, 0, "%s", line);
+        free(line);
 	attron(A_REVERSE);    
-	mvprintw(MAXPREVLINE + 1 - beginrow, 
-		pos_tab(parser->get_prevline(0), parser->get_tokenpos()) % x, "%s", token);
+	printw("%s", token);
 	attroff(A_REVERSE);
     
 	mvprintw(MAXPREVLINE + 2, 0, "\n");
@@ -596,13 +606,13 @@ void dialogscreen(TextParser * parser, char * token,
 
  // for terminal interface
 int dialog(TextParser * parser, Hunspell * pMS, char * token, char * filename,
-	char ** wlst, int ns) {
+	char ** wlst, int ns, int forbidden) {
     char buf[MAXLNLEN]; 
     char * buf2;
     wordlist * dicwords = NULL;
     int c;
 
-    dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+    dialogscreen(parser, token, filename, forbidden, wlst, ns);
 
     char firstletter='\0';
     
@@ -659,7 +669,7 @@ printw(gettext("\n-- Type space to continue -- \n"));
 	    while (getch()!=' ');
 	}
 	case 12: {
-    	    dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+    	    dialogscreen(parser, token, filename, forbidden, wlst, ns);
 	    break;
 	}
 	default: {
@@ -683,7 +693,7 @@ printw(gettext("\n-- Type space to continue -- \n"));
 
 		if ((!temp) || (temp[0] == '\0')) {
 		    free(temp);
-		    dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+		    dialogscreen(parser, token, filename, forbidden, wlst, ns);
 		    break;
 		}
 
@@ -775,7 +785,7 @@ printw(gettext("\n-- Type space to continue -- \n"));
 		    initscr();
 		    cbreak();
 #endif
-		    dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+		    dialogscreen(parser, token, filename, forbidden, wlst, ns);
 		    break;
 		}
 
@@ -786,7 +796,7 @@ printw(gettext("\n-- Type space to continue -- \n"));
 		initscr();
 		cbreak();
 #endif
-		dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+		dialogscreen(parser, token, filename, forbidden, wlst, ns);
 		refresh();
 
 #ifdef HAVE_READLINE
@@ -803,7 +813,7 @@ printw(gettext("\n-- Type space to continue -- \n"));
 
 		if ((!temp) || (temp[0] == '\0')) {
 		    free(temp);
-		    dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+		    dialogscreen(parser, token, filename, forbidden, wlst, ns);
 		    break;
 		}
 
@@ -857,10 +867,10 @@ printw(gettext("\n-- Type space to continue -- \n"));
 		    }
 		    
 		} else {
-		    dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+		    dialogscreen(parser, token, filename, forbidden, wlst, ns);
 		    printw(gettext("Pattern word must be in the dictionary. Press any key!"));
 		    getch();
-		    dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+		    dialogscreen(parser, token, filename, forbidden, wlst, ns);
 		    break;
 		}
 		goto ki;
@@ -872,7 +882,7 @@ printw(gettext("\n-- Type space to continue -- \n"));
 		if (modified) {
 		    printw(gettext("Are you sure you want to throw away your changes? "));
 		    if (getch()==(gettext("y"))[0]) return -1;
-    		    dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), wlst, ns);
+    		    dialogscreen(parser, token, filename, forbidden, wlst, ns);
 		    break;		    
 		} else {
 		    return -1;
@@ -888,16 +898,16 @@ int interactive_line(TextParser * parser, Hunspell * pMS, char * filename, FILE
 {
 	char * token;
 	int dialogexit = 0;
-
+        int info;
 	while ((token=parser->next_token())) {
-		if (! pMS->spell(token)) {
-			dialogscreen(parser, token, filename, pMS->get_forbidden_compound(), NULL, 0); // preview
+		if (! pMS->spell(token, &info, NULL)) {
+			dialogscreen(parser, token, filename, (info & SPELL_FORBIDDEN), NULL, 0); // preview
 			char ** wlst = NULL;
 			int ns = pMS->suggest(&wlst,token);
 			if (ns==0) {
-				dialogexit = dialog(parser, pMS, token, filename, wlst, ns);
+				dialogexit = dialog(parser, pMS, token, filename, wlst, ns, (info & SPELL_FORBIDDEN));
 			} else {	    
-				dialogexit = dialog(parser, pMS, token, filename, wlst, ns);
+				dialogexit = dialog(parser, pMS, token, filename, wlst, ns, (info & SPELL_FORBIDDEN));
 			}
 			for (int j = 0; j < ns; j++) {
 				free(wlst[j]);
@@ -1051,8 +1061,7 @@ int main(int argc, char** argv)
 			fprintf(stderr,gettext("E-mail bug reports to: szoszablya-user@lists.mokk.bme.hu\n"));
 			exit(0);
 	        } else if ((strcmp(argv[i],"-vv")==0) || (strcmp(argv[i],"-v")==0) || (strcmp(argv[i],"--version")==0)) {
-			fprintf(stdout,gettext(HUNSPELL_HEADING));
-			fprintf(stdout,HUNSPELL_VERSION);
+			fprintf(stdout,gettext(HUNSPELL_PIPE_HEADING));
 			fprintf(stdout,"\n");
                         if (strcmp(argv[i],"-vv")!=0) {
 			    fprintf(stdout,"\n");
